"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCompiler = void 0;
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const path = tslib_1.__importStar(require("path"));
const webpack_1 = tslib_1.__importDefault(require("webpack"));
const clearConsole_1 = require("./clearConsole");
const formatWebpackMessages_1 = require("./formatWebpackMessages");
const processHandler_1 = require("./processHandler");
const env_1 = require("../util/env");
const { argv } = process;
const argvSeparatorIndex = argv.indexOf('--');
const appArgs = ~argvSeparatorIndex
    ? process.argv.slice(argvSeparatorIndex + 1, argv.length)
    : [];
function createCompiler(config, options) {
    let compiler;
    const controller = new processHandler_1.AppController(path.join(config.output.path, config.output.filename), appArgs);
    try {
        compiler = webpack_1.default(config);
    }
    catch (err) {
        console.log(chalk_1.default.red('Failed to compile.'));
        console.log();
        console.log(err.message || err);
        console.log();
        process.exit(1);
        throw err.message;
    }
    compiler.hooks.invalid.tap('compileInvalidate', () => tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (env_1.IS_INTERACTIVE) {
            clearConsole_1.clearConsole();
        }
        console.log('Compiling...');
        console.log();
        if (!options.noAutoStart) {
            yield controller.stopApp();
        }
    }));
    compiler.hooks.done.tap('compileDone', (stats) => {
        if (env_1.IS_INTERACTIVE) {
            clearConsole_1.clearConsole();
        }
        const statsJson = stats.toJson({ errors: true, warnings: true });
        const messages = formatWebpackMessages_1.formatWebpackMessages(statsJson);
        const isSuccessful = !messages.errors.length && !messages.warnings.length;
        if (isSuccessful) {
            const successMessage = chalk_1.default.green('Compiled successfully!');
            const time = chalk_1.default.dim(`(${statsJson.time}ms)`);
            console.log(`${successMessage} ${time}`);
            console.log();
        }
        if (messages.errors.length) {
            const collapsedErrorLength = messages.errors.length - 1;
            if (messages.errors.length > 1 && !options.noCollapse) {
                messages.errors.length = 1;
            }
            console.log(chalk_1.default.red('Failed to compile.\n'));
            console.log(messages.errors.join('\n\n'));
            if (collapsedErrorLength && !options.noCollapse) {
                console.log();
                console.log(chalk_1.default.red(collapsedErrorLength +
                    (collapsedErrorLength > 1 ? ' errors have' : ' error has') +
                    ' been collapsed.'));
            }
            return;
        }
        // Show warnings if no errors were found.
        if (messages.warnings.length) {
            console.log(chalk_1.default.yellow('Compiled with warnings.\n'));
            console.log(messages.warnings.join('\n\n'));
            console.log('\nSearch for the ' +
                chalk_1.default.cyan('keywords') +
                ' to learn more about each warning.');
            console.log('To ignore, add ' +
                chalk_1.default.yellow('// eslint-disable-next-line') +
                ' to the line before.\n');
        }
        // Start the app if there are no errors
        if (!messages.errors.length && !options.noAutoStart) {
            controller.runApp();
        }
    });
    return compiler;
}
exports.createCompiler = createCompiler;
//# sourceMappingURL=webpackCompiler.js.map