"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const path = tslib_1.__importStar(require("path"));
const typescript = tslib_1.__importStar(require("typescript"));
const webpack_1 = tslib_1.__importDefault(require("webpack"));
const formatWebpackMessages_1 = require("../lib/formatWebpackMessages");
const paths_1 = require("../lib/paths");
const env_1 = require("../util/env");
const fileSizeReporter_1 = require("./util/fileSizeReporter");
const getWebpackConfig_1 = require("./util/getWebpackConfig");
const prodBundlePath = path.join(paths_1.paths.appBuild, paths_1.paths.prodBundle);
const sizeBeforeBuild = fileSizeReporter_1.getBundleSize(prodBundlePath);
const build = () => {
    console.log(chalk_1.default.cyan('Creating an optimized production build...'));
    console.log(chalk_1.default.green('Using TypeScript v' + typescript.version));
    console.log();
    const compiler = webpack_1.default(getWebpackConfig_1.getWebpackConfig('production'));
    return new Promise((resolve, reject) => {
        compiler.run((err, stats) => {
            if (err) {
                return reject(err);
            }
            const messages = formatWebpackMessages_1.formatWebpackMessages(stats.toJson());
            if (messages.errors.length) {
                if (messages.errors.length > 1 && !env_1.RuntimeOptions.noCollapse) {
                    messages.errors.length = 1;
                }
                return reject(new Error(messages.errors.join('\n\n')));
            }
            if (env_1.IS_CI && messages.warnings.length) {
                if (env_1.RuntimeOptions.bypassCiWarnings) {
                    console.log(chalk_1.default.yellow('\nBypassing warnings as CI errors due to --bypass-ci-warnings option.\n'));
                }
                else {
                    console.log(chalk_1.default.yellow('\nTreating warnings as errors because process.env.CI = true.\n' +
                        'Most CI servers set it automatically.\n'));
                    return reject(new Error(messages.warnings.join('\n\n')));
                }
            }
            return resolve({
                stats: stats,
                warnings: messages.warnings,
            });
        });
    });
};
build()
    .then(({ warnings }) => {
    if (warnings.length) {
        console.log(chalk_1.default.yellow('Compiled with warnings.\n'));
        console.log(warnings.join('\n\n'));
        console.log('\nSearch for the ' +
            chalk_1.default.cyan('keywords') +
            ' to learn more about each warning.');
        console.log('To ignore, add ' +
            chalk_1.default.yellow('// eslint-disable-next-line') +
            ' to the line before.\n');
    }
}, (err) => {
    console.log(chalk_1.default.red('Failed to compile.\n'));
    console.log(err.message);
    console.log();
    process.exit(1);
})
    .then(() => {
    const sizeAfterBuild = fileSizeReporter_1.getBundleSize(prodBundlePath);
    console.log();
    console.log(chalk_1.default.greenBright('Successfully built bundle.prod.js!'));
    console.log('Bundle size: ' + fileSizeReporter_1.diffFileSize(sizeBeforeBuild, sizeAfterBuild));
    console.log();
})
    .catch((err) => {
    if (err && err.message) {
        console.log(err.message);
    }
    process.exit(1);
});
//# sourceMappingURL=build.js.map