"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const paths_1 = require("../../../lib/paths");
const hash_1 = require("./hash");
const filesystem_1 = require("./filesystem");
const fs = tslib_1.__importStar(require("fs-extra"));
const child_process_1 = require("child_process");
const migrationLogger_1 = require("../../util/migrationLogger");
const fileExists_1 = require("../../util/fileExists");
const log = migrationLogger_1.createMigrationLogger();
let loggedInfo = false;
function handlePackageJson() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const packageJson = yield filesystem_1.loadJson(paths_1.paths.appPackageJson);
        const LEGACY_TSLINT_DEFAULT_COMMAND = 'tslint --project .';
        const ESLINT_DEFAULT_COMMAND = "eslint 'src/**/*.{js,ts}'";
        let shouldSave = false;
        if ('lint' in packageJson.scripts) {
            if (packageJson.scripts.lint === ESLINT_DEFAULT_COMMAND) {
                return;
            }
            if (packageJson.scripts.lint === LEGACY_TSLINT_DEFAULT_COMMAND) {
                log('Updating lint command in package.json', migrationLogger_1.MIGRATION_OP_TYPE.MODIFY);
                packageJson.scripts.lint = ESLINT_DEFAULT_COMMAND;
            }
            else {
                log('Found custom lint command, renaming script to "lint:legacy" in package.json', migrationLogger_1.MIGRATION_OP_TYPE.MODIFY);
                packageJson.scripts['lint:legacy'] = packageJson.scripts.lint;
                log('Adding new lint command in package.json', migrationLogger_1.MIGRATION_OP_TYPE.MODIFY);
                packageJson.scripts.lint = ESLINT_DEFAULT_COMMAND;
            }
            shouldSave = true;
        }
        else {
            log('Adding lint command to package.json', migrationLogger_1.MIGRATION_OP_TYPE.WRITE);
            packageJson.scripts.lint = ESLINT_DEFAULT_COMMAND;
            shouldSave = true;
        }
        if (shouldSave) {
            yield filesystem_1.saveJson(paths_1.paths.appPackageJson, packageJson);
        }
    });
}
function handleLinterConfigs() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const LEGACY_TSLINT_JSON_HASH = '3d6715a3490c843940026a410a0b41fa8c95ba468e93a8bab18fe0a44baada9c';
        const hasTslintFile = yield fileExists_1.fileExists(paths_1.paths.legacyAppTslint);
        const hasEslintFile = yield fileExists_1.fileExists(paths_1.paths.appEslint);
        const templateEslintrc = paths_1.resolveOwn('template/.eslintrc.json');
        let shouldAddEslintFile = true;
        if (hasEslintFile) {
            const sameEslintFile = (yield hash_1.getFileHash(paths_1.paths.appEslint)) ===
                (yield hash_1.getFileHash(templateEslintrc));
            if (sameEslintFile) {
                log('.eslintrc.json is already updated', migrationLogger_1.MIGRATION_OP_TYPE.SKIP);
                shouldAddEslintFile = false;
            }
            else {
                log('Renaming existing .eslintrc.json to .eslintrc.json.legacy', migrationLogger_1.MIGRATION_OP_TYPE.MODIFY);
                // TODO: possible collision
                fs.moveSync(paths_1.paths.appEslint, paths_1.paths.appEslint + '.legacy');
            }
        }
        if (shouldAddEslintFile) {
            log('Adding .eslintrc.json', migrationLogger_1.MIGRATION_OP_TYPE.WRITE);
            fs.copyFileSync(templateEslintrc, paths_1.paths.appEslint);
        }
        if (hasTslintFile) {
            const currentTslintHash = yield hash_1.getFileHash(paths_1.paths.legacyAppTslint);
            if (currentTslintHash === LEGACY_TSLINT_JSON_HASH) {
                log('Removing legacy tslint.json', migrationLogger_1.MIGRATION_OP_TYPE.DELETE);
                fs.unlinkSync(paths_1.paths.legacyAppTslint);
            }
            else {
                log('Renaming existing tslint.json to tslint.json.legacy', migrationLogger_1.MIGRATION_OP_TYPE.MODIFY);
                log('Custom tslint.json detected. You might need to manually port over your configuration, or use a tool like typescript-eslint/tslint-to-eslint-config', migrationLogger_1.MIGRATION_OP_TYPE.INFO);
                loggedInfo = true;
                fs.moveSync(paths_1.paths.legacyAppTslint, paths_1.paths.legacyAppTslint + '.legacy');
            }
        }
    });
}
function handleLinterDependencies() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const packageJson = yield filesystem_1.loadJson(paths_1.paths.appPackageJson);
        const { devDependencies = {}, dependenecies = {} } = packageJson;
        if ('tslint' in devDependencies) {
            log('Removing tslint from devDependencies', migrationLogger_1.MIGRATION_OP_TYPE.DELETE);
            // @@mutation
            delete packageJson.devDependencies['tslint'];
        }
        if ('tslint' in dependenecies) {
            log('Removing tslint from dependencies', migrationLogger_1.MIGRATION_OP_TYPE.DELETE);
            // @@mutation
            delete packageJson.dependencies['tslint'];
        }
        // save changes to package.json
        yield filesystem_1.saveJson(paths_1.paths.appPackageJson, packageJson);
        const allDependencies = [
            ...Object.keys(devDependencies),
            ...Object.keys(dependenecies),
        ];
        const tslintDependents = [];
        for (const dep of allDependencies) {
            if (~dep.indexOf('tslint') && dep !== 'tslint') {
                tslintDependents.push(dep);
            }
        }
        if (tslintDependents.length) {
            log('The following dependencies are related to tslint, but were not removed:', migrationLogger_1.MIGRATION_OP_TYPE.INFO);
            for (const dep of tslintDependents) {
                log(`  ${dep}`, migrationLogger_1.MIGRATION_OP_TYPE.INFO);
            }
            log('You might want to manually remove them, and add the proper configuration to .eslintrc.json', migrationLogger_1.MIGRATION_OP_TYPE.INFO);
            loggedInfo = true;
        }
    });
}
function handleAddDependencies() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const devDependencies = [
            'eslint',
            'eslint-plugin-import',
            'babel-eslint',
            '@typescript-eslint/eslint-plugin',
            '@typescript-eslint/parser',
        ];
        const hasYarnLockfile = yield fileExists_1.fileExists(paths_1.resolveApp('yarn.lock'));
        let cmd;
        let devDependencyArgs;
        if (hasYarnLockfile) {
            cmd = 'yarnpkg';
            devDependencyArgs = ['add', '--dev', '--silent', ...devDependencies];
        }
        else {
            cmd = 'npm';
            devDependencyArgs = [
                'install',
                '--silent',
                '--save-dev',
                ...devDependencies,
            ];
        }
        log('Installing ' + [...devDependencies].map((i) => chalk_1.default.green(i)).join(', '), migrationLogger_1.MIGRATION_OP_TYPE.INSTALL);
        // install dev dependencies
        const devDependencyProc = child_process_1.spawnSync(cmd, devDependencyArgs, {
            stdio: 'inherit',
            cwd: paths_1.resolveApp('.'),
        });
        if (devDependencyProc.status !== 0) {
            console.error();
            console.error(chalk_1.default.red('Command'), chalk_1.default.redBright(`'${cmd} ${devDependencyArgs.join(' ')}'`), chalk_1.default.red('failed.'));
            console.error();
            process.exit(1);
        }
        else {
            log('Successfully installed packages', migrationLogger_1.MIGRATION_OP_TYPE.INSTALL);
        }
    });
}
function main() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            yield handlePackageJson();
            yield handleLinterConfigs();
            yield handleLinterDependencies();
            yield handleAddDependencies();
            console.log();
            if (loggedInfo) {
                log('There are a few info messages logged above which might need to be addressed', migrationLogger_1.MIGRATION_OP_TYPE.INFO);
            }
            log('Successfully migrated tslint configuration to eslint configuration', migrationLogger_1.MIGRATION_OP_TYPE.DONE);
        }
        catch (e) {
            console.log();
            log('Uh oh, something went wrong. Please open an issue if this problem persists.', migrationLogger_1.MIGRATION_OP_TYPE.ERROR);
            log(e.message, migrationLogger_1.MIGRATION_OP_TYPE.ERROR);
            log(e.stack, migrationLogger_1.MIGRATION_OP_TYPE.ERROR);
        }
    });
}
main();
//# sourceMappingURL=index.js.map